/****************************************************************************
**
** DCOP Stub Implementation created by dcopidl2cpp from ksayit.kidl
**
** WARNING! All changes made in this file will be lost!
**
*****************************************************************************/

#include "ksayit_stub.h"
#include <dcopclient.h>

#include <kdatastream.h>


KSayItApp_stub::KSayItApp_stub( const QCString& app, const QCString& obj )
  : DCOPStub( app, obj )
{
}

KSayItApp_stub::KSayItApp_stub( DCOPClient* client, const QCString& app, const QCString& obj )
  : DCOPStub( client, app, obj )
{
}

KSayItApp_stub::KSayItApp_stub( const DCOPRef& ref )
  : DCOPStub( ref )
{
}

void KSayItApp_stub::dcopSayText( QString arg0 )
{
    if ( !dcopClient()  ) {
	setStatus( CallFailed );
	return;
    }
    QByteArray data;
    QDataStream arg( data, IO_WriteOnly );
    arg << arg0;
    dcopClient()->send( app(), obj(), "dcopSayText(QString)", data );
    setStatus( CallSucceeded );
}

void KSayItApp_stub::dcopStop()
{
    if ( !dcopClient()  ) {
	setStatus( CallFailed );
	return;
    }
    QByteArray data;
    dcopClient()->send( app(), obj(), "dcopStop()", data );
    setStatus( CallSucceeded );
}

void KSayItApp_stub::dcopSayXmlFile( KURL arg0 )
{
    if ( !dcopClient()  ) {
	setStatus( CallFailed );
	return;
    }
    QByteArray data;
    QDataStream arg( data, IO_WriteOnly );
    arg << arg0;
    dcopClient()->send( app(), obj(), "dcopSayXmlFile(KURL)", data );
    setStatus( CallSucceeded );
}


