/****************************************************************************
**
** DCOP Stub Definition created by dcopidl2cpp from ksayit.kidl
**
** WARNING! All changes made in this file will be lost!
**
*****************************************************************************/

#ifndef __KSAYIT_STUB__
#define __KSAYIT_STUB__

#include <dcopstub.h>
#include <qstringlist.h>
#include <qclipboard.h>
#include <kapp.h>
#include <kmainwindow.h>
#include <kpopupmenu.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>
#include <dcopobject.h>
#include <config.h>


class KSayItApp_stub : virtual public DCOPStub
{
public:
    KSayItApp_stub( const QCString& app, const QCString& id );
    KSayItApp_stub( DCOPClient* client, const QCString& app, const QCString& id );
    explicit KSayItApp_stub( const DCOPRef& ref );
    virtual ASYNC dcopSayText( QString msg );
    virtual ASYNC dcopStop();
    virtual ASYNC dcopSayXmlFile( KURL url );
protected:
    KSayItApp_stub() : DCOPStub( never_use ) {};
};


#endif
